# -*- coding: utf-8 -*
import numpy

from flyai.source.source import Source
from flyai.utils.yaml_helper import Yaml


class Dataset:
    def __init__(self, transformation=None, source=Source(),
                 processor_pka="processor",
                 processor_class=None):
        self.db = source
        self.model = Yaml().processor()
        self.x_test = None
        self.y_test = None
        self.test_trans = False
        if processor_class is not None:
            clz = processor_class
        else:
            clz = self.model['processor']
        self.processor = self.create_instance(processor_pka, clz)
        if transformation is not None:
            self.transformation = transformation
        else:
            try:
                self.transformation = self.create_instance('transformation', 'Transformation')
            except:
                self.transformation = None

    def next_batch(self, size=128, test_data=True):
        x_train, y_train, x_test, y_test = self.db.next_batch(size)
        x_train = self.processor_x(x_train)
        y_train = self.processor_y(y_train)
        if test_data:
            if self.x_test is None:
                self.x_test = self.processor_x(x_test)
            if self.y_test is None:
                self.y_test = self.processor_y(y_test)
        else:
            self.x_test = None
            self.y_test = None
        if self.transformation is not None:
            if self.test_trans:
                x_train, y_train, _, _ = self.transformation.transformation_data(x_train, y_train)
            else:
                x_train, y_train, self.x_test, self.y_test = self.transformation.transformation_data(x_train, y_train,
                                                                                                     self.x_test,
                                                                                                     self.y_test)
                self.test_trans = True
        return x_train, y_train, self.x_test, self.y_test

    def evaluate_data(self, path=None):
        if path is None:
            x_test, y_test = self.db.get_validation_data()
        else:
            x_test, y_test = self.db.get_evaluate_data(path)
        x_test = self.processor_x(x_test)
        y_test = self.processor_y(y_test)
        if self.transformation is not None:
            _, _, x_test, y_test = self.transformation.transformation_data(x_test=x_test, y_test=y_test)
        return x_test, y_test

    def evaluate_source(self, path):
        x_test, y_test = self.db.get_evaluate_data(path)
        return x_test, y_test

    def get_all_data(self):
        x_train, y_train, x_test, y_test = self.db.get_all_data()
        return x_train, y_train, x_test, y_test

    def get_all_processor_data(self):
        x_train, y_train, x_test, y_test = self.get_all_data()
        x_train = self.processor_x(x_train)
        y_train = self.processor_y(y_train)
        x_test = self.processor_x(x_test)
        y_test = self.processor_y(y_test)
        if self.transformation is not None:
            x_train, y_train, x_test, y_test = self.transformation.transformation_data(x_train, y_train, x_test, y_test)
        return x_train, y_train, x_test, y_test

    def processor_x(self, x_datas):
        processors = []
        init = False
        processor_len = 0
        for data in x_datas:
            processor = self.get_method_dict(self.processor, self.model['input_x'], **data)
            if not isinstance(processor, tuple):
                processor_len = 1
            if processor_len == 1:
                processors.append(numpy.array(processor))
            else:
                if not init:
                    processors = [[] for i in range(len(processor))]
                    init = True
                index = 0
                for item in processor:
                    processors[index].append(numpy.array(item))
                    index += 1
        if processor_len == 1:
            return numpy.concatenate([processors], axis=0)
        else:
            list = []
            for column in processors:
                list.append(numpy.concatenate([column], axis=0))
            return list

    def processor_y(self, y_datas):
        processors = []
        init = False
        processor_len = 0
        # for data in y_datas:
        #     processor = self.get_method_dict(self.processor, self.model['input_y'], **data)
        #     processors.append(numpy.array(processor))
        # return numpy.concatenate([processors], axis=0)
        for data in y_datas:
            processor = self.get_method_dict(self.processor, self.model['input_y'], **data)
            if not isinstance(processor, tuple):
                processor_len = 1
            if processor_len == 1:
                processors.append(numpy.array(processor))
            else:
                if not init:
                    processors = [[] for i in range(len(processor))]
                    init = True
                index = 0
                for item in processor:
                    processors[index].append(numpy.array(item))
                    index += 1
        if processor_len == 1:
            return numpy.concatenate([processors], axis=0)
        else:
            list = []
            for column in processors:
                list.append(numpy.concatenate([column], axis=0))
            return list

    def predict_data(self, **data):
        processors = []
        processor = self.get_method_dict(self.processor, self.model['output_x'], **data)
        processor_len = 0
        if not isinstance(processor, tuple):
            processors.append(numpy.array(processor))
            processor_len = 1
        else:
            processors = [[] for i in range(len(processor))]
            index = 0
            for item in processor:
                processors[index].append(numpy.array(item))
                index += 1
        if processor_len == 1:
            x = numpy.concatenate([processors], axis=0)
        else:
            list = []
            for column in processors:
                list.append(numpy.concatenate([column], axis=0))
            x = list
        if self.transformation is not None:
            x, _, _, _ = self.transformation.transformation_data(x)
        return x

    def to_categorys(self, predict):
        return self.get_method_list(self.processor, self.model['output_y'], predict)

    def create_instance(self, module_name, class_name, *args, **kwargs):
        module_meta = __import__(module_name, globals(), locals(), [class_name])
        class_meta = getattr(module_meta, class_name)
        return class_meta(*args, **kwargs)

    def get_method_dict(self, clz, method_name, **args):
        m = getattr(clz, method_name)
        return m(**args)

    def get_method_list(self, clz, method_name, *args):
        m = getattr(clz, method_name)
        return m(*args)
